<?php
// $Id: flickrrippr.inc,v 1.22 2010/09/02 00:23:13 taniwha Exp $

/**
 * @file flickrriprr.inc 
 * Functions that are likely to be of use to other modules, are in this file.
 * PHP Flickr: http://phpflickr.com/docs/
 * Flickr API docs http://www.flickr.com/services/api/
 */
//
//
///**
// * Works out img tags to display a photo - optionally wrapped in a link to the photo's page.
// * NOTE: flickr requires links to flickr.com if you're not caching the file on your own server
// */
/**
* Helper function, to get drupal user details for a flickr user (string of their username of flickr)
*/
function flickrrippr_get_user_by_flickrusername($fuser) {
  if (!$fuser) {
    drupal_set_message(t("Refusing to search for blank username"));
    return;
  }
  
  $flickruser = db_fetch_object(db_query("SELECT * FROM {flickrusers} WHERE LOWER(flickrusername) = '%s'", strtolower($fuser)));
  if (!$flickruser) {
    drupal_set_message(t("Flickruser %flickrusername is not a registered user of this site", array('%flickrusername' => $fuser)));
    return;
  }
  
  $user = user_load(array('uid' => $flickruser->uid));
  return $user;
}

function flickrrippr_get_user_by_nsid($nsid) {
    if (!$nsid) {
    drupal_set_message(t("Refusing to search for blank nsid"));
    return;
  }
  
  $flickruser = db_fetch_object(db_query("SELECT * FROM {flickrusers} WHERE flickrid = '%s'", $nsid));
  if (!$flickruser) {
    drupal_set_message(t("NSID %nsid is not a registered user of this site.", array('%nsid' => $nsid)));
    return;
  }
  
  $user = user_load(array('uid' => $flickruser->uid));
  return $user;
}


/**
* Work out the path to a photo - also handles the cache on the local filesystem
*/
function flickrrippr_path($flickr_photo, $cache = TRUE, $size = FALSE) {


 //http://farm{farm-id}.static.flickr.com/{server-id}/{id}_{secret}.jpg
  $src =  'http://farm'. $flickr_photo['farm'] .'.static.flickr.com/'. $flickr_photo['server'] .'/'. $flickr_photo['id'] .'_'. $flickr_photo['secret'] . ($size ? '_'. $size : '') .'.jpg';
  
  if (!$cache) {
    return $src;
  }
  return $src;

 $folder =  file_directory_path() . '/flickr';
 if (!file_create_path($folder)) {
   drupal_set_message(t('Files directory not set, unable to cache flickr photos'), 'error');
   return;
 }
  
  $filename =  $folder .'/'. $flickr_photo['id'] .'_'. $flickr_photo['secret'] . ($size ? '_'. $size : '') .'.jpg';
  
 //If we've already go it, then we dont' need to get it
  if (file_exists($filename)) {
   //if (filesize($filename) > 0) {
     //TODO, check mtime against photo info -- if possible
    return $filename;
   //}
  }
  
  require_once "HTTP/Request.php";
  $req =& new HTTP_Request($src);
   //Something went wrong?
  if (PEAR::isError($req->sendRequest())) {
    drupal_set_message(t("Failed to retrieve jpeg file from flick.com, url = %url", array('%url' => $src)));
   //use flickr hosted jpeg
    return $src;
  }
  
   //file handle to save the jpeg into
  $contents = $req->getResponseBody();
  if (file_save_data($contents, $filename, FILE_EXISTS_REPLACE)) {
    return $filename;
  }
  else {
    drupal_set_message(t("Failed to save image to %filename", array('%filename' => $filename)));
    return $src;
  }
 
}

/**
 * Makes a node for a flickr photo - or updates if it already exists
 */
function flickrrippr_makenode($photo_id) {


  if (!$photo_id) {
    drupal_set_message(t("Photo_id required"), 'error');
    return;
  }

  drupal_set_message(t("Fetching data from flickr .com for photoid=%photo_id", array('%photo_id' => $photo_id)));
// 
  $nid = flickrrippr_photo_get_nid($photo_id) ;
  if (!empty($nid)) {
//     //already exists
    $node = node_load(array('nid' => $nid));
//     drupal_set_message(t('Existing node id = !nid. !link.', array('!nid' => $nid, '!link' => l(check_plain($node->title), 'node/'. $nid))));
  }
  else {
  
  }

  $flickr = flickrapi_phpFlickr();
  if (!$flickr) {
    drupal_set_message(t("Unable to query flickr.com, library missing"));
    return $output;
  }

  //Info to put into the node
  $flickr_info = $flickr->photos_getInfo($photo_id);
  //we can't get info from flickr -- mark this node as bad
  if (!$flickr_info) {
    drupal_set_message(t('No info found on photo %photoid.', array('%photoid' => $photo_id)), 'error');
    db_query("UPDATE {flickrphotos} SET failed = 1 WHERE flickrphoto_id = %d", $photo_id);
    node_save($node);
    return;
  }
  
  $flickr_context = $flickr->photos_getContext($photo_id);
  

  //flickr username
  $flickrusername = $flickr_info['owner']['username'];
  $flickrid = $flickr_info['owner']['nsid'];
  
//   drupal_set_message(t("Making a node for photo by %flickrusername", array('%flickrusername' => $flickrusername)), 'error');

  //get user details
//   $user = flickrrippr_get_user_by_flickrusername($flickrusername);
  $user = flickrrippr_get_user_by_nsid($flickrid);
  if (!$user) {
    drupal_set_message(t("User %flickrusername doesn't exist, unable to save photo %photo_id", array('%flickrusername' => $flickrusername, '%photo_id' => $photo_id)), 'error');
    return FALSE;
  }

  //Full (normal) sized image
  $photo_path =  flickrrippr_path($flickr_info);
  $thumbnail_path =  flickrrippr_path($flickr_info, TRUE, 'm');
  $description = $flickr_info['description'];
  $title = $flickr_info['title'];
  $page_url = $flickr_info['urls']['url'][0]['_content'];

  
  $node->title = htmlentities($title);
  $node->created = strtotime($flickr_info['dates']['taken']);
//   $node->name = $user->name;
  $node->uid = $user->uid;
  $node->type = 'flickrrippr_photo';
//   $node->published = 1;
//   $node->status =1;
//   $node->comment = 2;
//   $node->promote = 1;
//   $node->format = 2;

  node_save($node);

  if ($node->is_new) {
//     drupal_set_message("New node, nid = $node->nid");
    $result = db_query("INSERT INTO {flickrphotos} (flickrphoto_id, nid, lastfetched, photo_path, thumbnail_path, description, page_url)
      VALUES ('%s', %d, %d, '%s', '%s', '%s', '%s')",
      $photo_id, $node->nid, strtotime('now'), $photo_path, $thumbnail_path, $description, $page_url);
      drupal_set_message(t("Saved new photo %photo_id, ". l($node->title, 'node/' . $node->nid), array('%photo_id' => $photo_id)));
  }
  else {
    $result = db_query("UPDATE {flickrphotos} SET
      lastfetched=%d,
      photo_path='%s',
      thumbnail_path='%s',
      description = '%s',
      page_url = '%s'
      WHERE flickrphoto_id=%d AND nid=%d",
      strtotime('now'),
      $photo_path,
      $thumbnail_path,
      $description,
      $page_url,
      $photo_id,
      $node->nid);
    drupal_set_message(t("Updated photo %photo_id, ". l($node->title, 'node/' . $node->nid), array('%photo_id' => $photo_id)));
  }

  if (variable_get('flickrrippr_import_tags', FALSE)) {
    include_once(drupal_get_path('module', 'flickrrippr') .'/flickrrippr.tags.inc');
    $vocab = variable_get('flickrrippr_tags_vocab', 0);
    if (!$vocab) {
      return;
    }

    flickrrippr_photo_add_tags($nid, $flickr_info);
  }

}



/**
 * get the node id for a flickr photo id
 */
function flickrrippr_photo_get_nid($photo_id) {
  $rec = db_fetch_object(db_query("SELECT * FROM {flickrphotos} WHERE flickrphoto_id = '%s'", $photo_id));
  return (!empty($rec->nid) ? $rec->nid : NULL);
}